/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is NewsFox.
 *
 * The Initial Developer of the Original Code is
 * Andy Frank <andy@andyfrank.com>.
 * Portions created by the Initial Developer are Copyright (C) 2005-2006
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Andrey Gromyko <andrey@gromyko.name>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the LGPL or the GPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

////////////////////////////////////////////////////////////////
// Global
////////////////////////////////////////////////////////////////

var checkInProgress = false;
var cancelCheck = false;
var refreshingAllFeeds = false;
var httpRequestTimeout;
var xmlhttp = null;
var currentRequestFeedNo = -1;

////////////////////////////////////////////////////////////////
// Check feeds
////////////////////////////////////////////////////////////////

function doCheckFeeds()
{
  // Bail out if check already in progress
  if (checkInProgress) return;
  if (model.size() > 0) 
  {
    checkInProgress = true;
    var elem = document.getElementById("busyAnimation");
    elem.style.visibility = "visible";
    precheckFeed(0);
  }
}

function doCancelCheckFeeds()
{
  if (checkInProgress) cancelCheck = true;
}

function precheckFeed(index)
{
  if( index >= model.size() )
  {
    if( checkInProgress )
      postRefresh();
    return;
  }

  var elem = document.getElementById("busyTextNumbers");
  elem.value = (index+1) + " / " + model.size();

  var feed = model.get(index);
  feed.error = ERROR_OK;

  if( refreshingAllFeeds && !feed.autoCheck ) // then we skip
  {
    precheckFeed(index+1);
  }
  else
  {
    xmlhttp = new XMLHttpRequest();
		var url = feed.url;
		// trick of adding the current time to bypass the cache; suggested by Konstantin Svist
		url += (url.match(/\?/) == null ? '?' : '&') + (new Date()).getTime();
    xmlhttp.open("get", url);
    xmlhttp.setRequestHeader("User-Agent", "Mozilla/5.0 NewsFox/0.6");
    xmlhttp.overrideMimeType("application/xml");
    // TODO: do error handling
    // xmlhttp.onerror = foo;
    xmlhttp.onload = function() { checkFeed(index, xmlhttp.responseXML); }

		currentRequestFeedNo = index;
		httpRequestTimeout = setTimeout(this.abortHttpRequest, 180000); // 3 minutes
    // the next line doesn't have any effect :( Any other ideas how to bypass the cache?
    // xmlhttp.channel.loadFlags = Components.interfaces.nsICachingChannel.LOAD_BYPASS_CACHE;
    xmlhttp.send(null);
  }
}

function checkFeed(index, xml)
{
	if(null != httpRequestTimeout)
		clearTimeout(httpRequestTimeout);
  try
  {
    var feed = model.get(index);
    if (!feed.loaded)
    {
      loadFeed(feed);
      feed.loaded = true;
    }

		var lastNewestEntryDate;
		var feedSize = feed.size();
		var i = 0;
		var art;
		if( feedSize > 0 )
		{
			art = feed.get(0);
			lastNewestEntryDate = art.date;
		}
		for (i=1; i<feedSize; i++)
		{
			art = feed.get(i);
			if( art.date > lastNewestEntryDate )
				lastNewestEntryDate = art.date;
		}

		var newItemsCount = 0;
    var parser = new Parser(xml);
    if (parser.title != null) 
      feed.defaultName = encodeString(parser.title);
    if (parser.link != null)
      feed.homepage = encodeUrl(parser.link);
    var oldestEntryDate = new Date();

    for (i=0; i<parser.items.length; i++)
    {
      var item = parser.items[i];
      if ("" != item.pubDate)
			{
				if (oldestEntryDate > item.pubDate)
					oldestEntryDate = item.pubDate;
				if( lastNewestEntryDate < item.pubDate )
					newItemsCount++;
			}

      if (doesArticleExist(feed, item.link)) continue;

      art = new Article();
      art.title = encodeString(item.title);
      art.link = item.link;
      art.date = item.pubDate;
      art.body = encodeString(item.content);
      art.read = false;
      art.flag = false;
      // AG: added category
      art.category = encodeString(item.category);

      if (art.date == "") art.date = new Date();
      if (art.title == "") art.title = (item.content != "") ? encodeString(item.content.substr(0, 70)) + "..." : "...";

      feed.add(art);
    }

    feed.sort();
    feed.sortCategories();

    // AG: delete old not flagged entries if needed, optionally leaving unread
    if (feed.deleteOld)
    {
      feedSize = feed.size();
      for (i=0; i<feedSize; i++)
      {
        art = feed.get(i);
        if ((art.date < oldestEntryDate) && !feed.isFlagged(art))
        {
					if( !feed.dontDeleteUnread || feed.isRead(art) )
					{
						feed.remove(i--);
						feedSize--;
					}
        }
      }
    }

		feed.setNewItemsCount(newItemsCount);
    saveFeed(feed);
  }
  catch (err) 
  { 
    feed.error = err;
    feed.error += "<p><a href='" + feed.url + "'>" + feed.url + "</a>";
  }

  // Update FeedTree
  var tree = document.getElementById("newsfox.feedTree");
  // by AG tree.treeBoxObject.invalidateRow(index);
  var curSelection = tree.currentIndex;
  tree.view = new FeedTreeModel(); // AG: to handle categories
	if( curSelection > -1 )
  	tree.view.selection.select(curSelection);

  if (!cancelCheck && index < model.size()-1)
  {
    // Check next feed
    precheckFeed(index+1);
  }
  else
  {
    // We're done!
    postRefresh();
  }
}

function abortHttpRequest(feedNo)
{
	if( currentRequestFeedNo > -1 )
	{
		xmlhttp.abort();
		precheckFeed(currentRequestFeedNo+1);
	}
}

function postRefresh()
{
  var elem = document.getElementById("busyAnimation");
  elem.style.visibility = "hidden";
  checkInProgress = false;
  cancelCheck = false;
  refreshingAllFeeds = false;
  saveFeedModel();
  feedSelected();
	if( options.autoRefresh )
		autoRefreshTimer = setTimeout(this.checkFeeds, options.autoRefreshInterval * 60 * 1000);
	if( options.notifyUponNew )
		reportRefreshResults();
}

function reportRefreshResults()
{
	var newTotalCount = 0;
	var unreadTotalCount = 0;
	for(var i=0; i < model.size(); i++)
	{
		var feed = model.get(i);
		newTotalCount += feed.getNewItemsCount();
		unreadTotalCount += feed.getUnread(0);
	}
	if( newTotalCount > 0 )
	{
		var stringsBundle = document.getElementById("newsfox-string-bundle");
		var strNew = stringsBundle.getString('alert.new');
		var strUnread = stringsBundle.getString('alert.unread');
		var message = newTotalCount + " " + strNew + ", " + unreadTotalCount + " " + strUnread;
		var alerts = Components.classes["@mozilla.org/alerts-service;1"]
                          .getService(Components.interfaces.nsIAlertsService);
		alerts.showAlertNotification("chrome://newsfox/skin/newsfox-32.png", "NewsFox", message, false, "", null);
	}
}

////////////////////////////////////////////////////////////////
// Util
////////////////////////////////////////////////////////////////

/**
 * Encode unicode characters to XML-friendly format.
 */
function encodeString(s) 
{
	if( !s ) return "";
  for (var i=0; i<s.length; i++)
  {
    var code = s.charCodeAt(i);
    if (code < 32 || code > 126)
    { 
      // Replacement character
      var replace = "&#" + code + ";";
      // Ignore newline characters and tabs
      // TODO - should we ignore everything < 32 ?
      if (code == 9 || code == 10) replace = " ";
      s = s.substring(0, i) + replace + s.substring(i+1);
    }
  }
  return s;
}

/**
 * Encode url problem charaters.
 */
function encodeUrl(s)
{
  s = s.replace(new RegExp('&','gi'), '&amp;');
  return s;
}

/**
 * Return true if this article already exists.
 * We assume that every article has a unique link.
 */
function doesArticleExist(feed, link)
{
  for (var i=0; i<feed.size(); i++)
    if (feed.get(i).link == link) 
      return true;
  return false;
}
